#!/bin/bash -e
#
#  Common functions for UOSCI jobs, specific to OpenStack actions.

function f_openstack_uos_env_var_normalize() {
# Ensure that both representations of the Ubuntu Release x OpenStack Release
# environment variables are set.
  if [[ -z "$UBUNTU_RELEASE" ]] && [[ -z "$OPENSTACK_RELEASE" ]] && [[ -n "$U_OS" ]]; then
    echo " . Using U_OS to determine UBUNTU_RELEASE and OPENSTACK_RELEASE."
    export UBUNTU_RELEASE=$(echo $U_OS | cut -f 1 -d "-")
    export MOJO_SERIES=$UBUNTU_RELEASE
    export OPENSTACK_RELEASE=$(echo $U_OS | cut -f 2 -d "-")

  elif [[ -n "$UBUNTU_RELEASE" ]] && [[ -n "$OPENSTACK_RELEASE" ]] && [[ -z "$U_OS" ]]; then
    echo " . Using UBUNTU_RELEASE and OPENSTACK_RELEASE to determine U_OS."
    export U_OS="${UBUNTU_RELEASE}-${OPENSTACK_RELEASE}"
    export MOJO_SERIES=$UBUNTU_RELEASE

  elif [[ -n "$UBUNTU_RELEASE" ]] && [[ -n "$OPENSTACK_RELEASE" ]] && [[ -n "$U_OS" ]]; then
    echo " . UBUNTU_RELEASE, OPENSTACK_RELEASE and U_OS are all set."
    export MOJO_SERIES=$UBUNTU_RELEASE
  fi
}


function f_env_set_ldap_location() {
    f_line $FUNCNAME
    # Setup LDAP test fixture for keystone-ldap testing
    [[ -z "$TEST_LDAP_SERVER" ]] && export TEST_LDAP_SERVER="ldap://10.245.162.249/"
    [[ -z "$TEST_LDAP_USER" ]] && export TEST_LDAP_USER="cn=admin,dc=test,dc=com"
    [[ -z "$TEST_LDAP_PASSWORD" ]] && export TEST_LDAP_PASSWORD="crapper"
    [[ -z "$TEST_LDAP_SUFFIX" ]] && export TEST_LDAP_SUFFIX="dc=test,dc=com"
    echo "    TEST_LDAP_SERVER: $TEST_LDAP_SERVER"
    echo "    TEST_LDAP_USER: $TEST_LDAP_USER"
    echo "    TEST_LDAP_PASSWORD: $TEST_LDAP_PASSWORD"
    echo "    TEST_LDAP_SUFFIX: $TEST_LDAP_SUFFIX"
}


function f_env_set_amphora_location() {
    f_line $FUNCNAME
    # Set Octavia amphora test image location if not already set.
    [[ -z "$TEST_AMPHORA_LOCATION" ]] && export TEST_AMPHORA_LOCATION='http://10.245.161.162/swift/v1/images/test-only-amphora-x64-haproxy-ubuntu-xenial.qcow2'
    echo "    TEST_AMPHORA_LOCATION: $TEST_AMPHORA_LOCATION"
}


function f_openstack_set_network_env_vars() {
    f_line $FUNCNAME
    f_activate_clients
    f_subnet_id
    export TEST_NET_ID="$NETWORK_ID"  # Comes from oscirc
    export TEST_CIDR_EXT="$NETWORK_CIDR"  # Comes from oscirc
    export TEST_CIDR_END="$(ipcalc $TEST_CIDR_EXT | awk '/^HostMax: /{ print $2 }')"
    export TEST_GATEWAY="$(ipcalc $TEST_CIDR_EXT | awk '/HostMin:/{ print $2 }')"
    export TEST_NAME_SERVER=$(echo $TEST_GATEWAY | sed "s/\.1$/.2/g")

    # OSCI subnet dhcp pools end at the highest .199, so the
    # floating IP and VIP pools are safe in the last .200-254 of the network.
    export TEST_FIP_START=$(echo $TEST_CIDR_END | sed "s/254/200/g")
    export TEST_FIP_END="$(echo $TEST_CIDR_END | sed 's/254/229/g')"
    export TEST_FIP_RANGE="${TEST_FIP_START}:${TEST_FIP_END}"

    export TEST_VIP00=$(echo $TEST_CIDR_END | sed "s/254/230/g")
    export TEST_VIP01=$(echo $TEST_CIDR_END | sed "s/254/231/g")
    export TEST_VIP02=$(echo $TEST_CIDR_END | sed "s/254/232/g")
    export TEST_VIP03=$(echo $TEST_CIDR_END | sed "s/254/233/g")
    export TEST_VIP04=$(echo $TEST_CIDR_END | sed "s/254/234/g")
    export TEST_VIP05=$(echo $TEST_CIDR_END | sed "s/254/235/g")
    export TEST_VIP06=$(echo $TEST_CIDR_END | sed "s/254/236/g")
    export TEST_VIP07=$(echo $TEST_CIDR_END | sed "s/254/237/g")
    export TEST_VIP08=$(echo $TEST_CIDR_END | sed "s/254/238/g")
    export TEST_VIP09=$(echo $TEST_CIDR_END | sed "s/254/239/g")
    export TEST_VIP10=$(echo $TEST_CIDR_END | sed "s/254/240/g")
    export TEST_VIP11=$(echo $TEST_CIDR_END | sed "s/254/241/g")
    export TEST_VIP12=$(echo $TEST_CIDR_END | sed "s/254/242/g")
    export TEST_VIP13=$(echo $TEST_CIDR_END | sed "s/254/243/g")
    export TEST_VIP14=$(echo $TEST_CIDR_END | sed "s/254/244/g")
    export TEST_VIP15=$(echo $TEST_CIDR_END | sed "s/254/245/g")
    export TEST_VIP16=$(echo $TEST_CIDR_END | sed "s/254/246/g")
    export TEST_VIP17=$(echo $TEST_CIDR_END | sed "s/254/247/g")
    export TEST_VIP18=$(echo $TEST_CIDR_END | sed "s/254/248/g")
    export TEST_VIP19=$(echo $TEST_CIDR_END | sed "s/254/249/g")
    export TEST_VIP20=$(echo $TEST_CIDR_END | sed "s/254/250/g")
    export TEST_VIP=$TEST_VIP00
    export TEST_VIP_RANGE="${TEST_VIP00}:${TEST_VIP20}"

    # Preserve backward compatibility while deprecating non-prefixed and elder-prefixed env vars:
    export OS_VIP00=$TEST_VIP00
    export OS_VIP01=$TEST_VIP01
    export OS_VIP02=$TEST_VIP02
    export OS_VIP03=$TEST_VIP03
    export OS_VIP04=$TEST_VIP04
    export OS_VIP05=$TEST_VIP05
    export OS_VIP06=$TEST_VIP06
    export OS_VIP07=$TEST_VIP07
    export OS_VIP08=$TEST_VIP08
    export OS_VIP09=$TEST_VIP09
    export OS_VIP10=$TEST_VIP10
    export OS_VIP11=$TEST_VIP11
    export OS_VIP12=$TEST_VIP12
    export OS_VIP13=$TEST_VIP13
    export OS_VIP14=$TEST_VIP14
    export OS_VIP15=$TEST_VIP15
    export OS_VIP16=$TEST_VIP16
    export OS_VIP17=$TEST_VIP17
    export OS_VIP18=$TEST_VIP18
    export OS_VIP19=$TEST_VIP19
    export OS_VIP20=$TEST_VIP20
    export NAME_SERVER="${TEST_NAME_SERVER}"
    export NET_ID="${TEST_NET_ID}"
    export CIDR_EXT="${TEST_CIDR_EXT}"
    export GATEWAY="${TEST_GATEWAY}"
    export CIDR_END="${TEST_CIDR_END}"
    export FIP_START="${TEST_FIP_START}"
    export FIP_END="${TEST_FIP_END}"
    export FIP_RANGE="${TEST_FIP_RANGE}"
    export VIP_RANGE="${TEST_VIP_RANGE}"

    echo "  + Setting network env vars:"
    echo "    TEST_NET_ID: $TEST_NET_ID"
    echo "    TEST_GATEWAY: $TEST_GATEWAY"
    echo "    TEST_NAME_SERVER: $TEST_NAME_SERVER"
    echo "    TEST_CIDR_EXT: $TEST_CIDR_EXT"
    echo "    TEST_FIP_RANGE: $TEST_FIP_RANGE"
    echo "    TEST_VIP_RANGE: $TEST_VIP_RANGE"
    echo "    TEST_VIP: $TEST_VIP"
    echo "    TEST_VIP00: $TEST_VIP00"
    echo "    TEST_VIP01: $TEST_VIP01"
    echo "    TEST_VIP02: $TEST_VIP02"
    echo "    TEST_VIP03: $TEST_VIP03"
    echo "    TEST_VIP04: $TEST_VIP04"
    echo "    TEST_VIP05: $TEST_VIP05"
    echo "    TEST_VIP06: $TEST_VIP06"
    echo "    TEST_VIP07: $TEST_VIP07"
    echo "    TEST_VIP08: $TEST_VIP08"
    echo "    TEST_VIP09: $TEST_VIP09"
    echo "    TEST_VIP10: $TEST_VIP10"
    echo "    TEST_VIP11: $TEST_VIP11"
    echo "    TEST_VIP12: $TEST_VIP12"
    echo "    TEST_VIP13: $TEST_VIP13"
    echo "    TEST_VIP14: $TEST_VIP14"
    echo "    TEST_VIP15: $TEST_VIP15"
    echo "    TEST_VIP16: $TEST_VIP16"
    echo "    TEST_VIP17: $TEST_VIP17"
    echo "    TEST_VIP18: $TEST_VIP18"
    echo "    TEST_VIP19: $TEST_VIP19"
    echo "    TEST_VIP20: $TEST_VIP20"
    echo "    TEST_VIP_RANGE: $TEST_VIP_RANGE"
    f_env_set_mojo_env_vars
    f_env_set_ldap_location
    f_env_set_amphora_location
}


function f_openstack_get_zaza_github_func_test_pr() {
    # Magic to use an alternate zaza repo when specified in the gerrit commit message
    f_line $FUNCNAME
    if [ -z "$GERRIT_CHANGE_COMMIT_MESSAGE" ]; then
        echo "No Gerrit commit message found"
        return
    fi
    _pr=$( echo -n $GERRIT_CHANGE_COMMIT_MESSAGE | base64 -d | \
          awk 'tolower($0) ~ /func-test-pr: .*github.*zaza.*pull/ {print $2; exit;}' | \
          rev | cut -sf1,3,4 -d/ | rev)
    if [ -z "$_pr" ]; then
      echo "No GitHub Func-Test-PR found..."
      return
    fi
    _pr_repo=$(echo $_pr | cut -f1-2 -d/)
    _pr_num=$(echo $_pr | cut -f3 -d/)
    _github_api_pr_url="https://api.github.com/repos/$_pr_repo/pulls/$_pr_num"
    _pr_label=$(curl $_github_api_pr_url | \
                    awk '/"head":/ {getline; gsub(/"|,/, ""); print $2}')
    export GITHUB_PR_REPO=$(echo $_pr_repo | cut -f2 -d/)
    export GITHUB_PR_ORG=$(echo $_pr_repo | cut -f1 -d/)
    export GITHUB_PR_LOGIN=$(echo $_pr_label | cut -f1 -d:)
    export GITHUB_PR_BRANCH=$(echo $_pr_label | cut -f2 -d: | \
                                 sed -e 's/\//\\\//')
}


function f_openstack_update_zaza_test_requirements() {
    # Magic to use an alternate zaza repo when specified in the gerrit commit message
    f_line $FUNCNAME
    f_openstack_get_zaza_github_func_test_pr
    if [ -n "$GITHUB_PR_LOGIN" -a -n "$GITHUB_PR_BRANCH" ]; then
      sed -i s/${GITHUB_PR_ORG}\\/${GITHUB_PR_REPO}.git/${GITHUB_PR_LOGIN}\\/${GITHUB_PR_REPO}.git@${GITHUB_PR_BRANCH}/ ${BUILT_ASSET_DIR-$CO_DIR}/test-requirements.txt || true
      sed -i s/${GITHUB_PR_ORG}\\/${GITHUB_PR_REPO}.git/${GITHUB_PR_LOGIN}\\/${GITHUB_PR_REPO}.git@${GITHUB_PR_BRANCH}/ ${BUILT_ASSET_DIR-$CO_DIR}/src/test-requirements.txt || true
    fi
}


function f_openstack_set_zaza_env_vars() {
    # TODO: Consider removing.  This was necessary when nova-lxd
    # compute nodes were in the CI test infra, but that is no longer
    # the case, and the charm-test-infra default logic 'should' stand.
    f_line $FUNCNAME
    export MODEL_CONSTRAINTS="virt-type=kvm"
    echo "  + Setting zaza env vars:"
    echo "    MODEL_CONSTRAINTS: $MODEL_CONSTRAINTS"
}


function f_env_set_network_env_vars() {
    f_line $FUNCNAME
    f_openstack_set_network_env_vars
    f_openstack_set_zaza_env_vars
}


function f_env_set_mojo_env_vars() {
    # TODO: Deprecate the use of MOJO_* vars in favor of TEST_* env vars.
    export MOJO_NAME_SERVER="${TEST_NAME_SERVER}"
    export MOJO_NET_ID="${TEST_NET_ID}"
    export MOJO_SUBNET_ID="${TEST_SUBNET_ID}"
    export MOJO_CIDR_EXT="${TEST_CIDR_EXT}"
    export MOJO_GATEWAY="${TEST_GATEWAY}"
    export MOJO_FIP_RANGE="${TEST_FIP_RANGE}"
    export MOJO_OS_VIP00="${TEST_VIP00}"
    export MOJO_OS_VIP01="${TEST_VIP01}"
    export MOJO_OS_VIP02="${TEST_VIP02}"
    export MOJO_OS_VIP03="${TEST_VIP03}"
    export MOJO_OS_VIP04="${TEST_VIP04}"
    export MOJO_OS_VIP05="${TEST_VIP05}"
    export MOJO_OS_VIP06="${TEST_VIP06}"
    export MOJO_OS_VIP07="${TEST_VIP07}"
    export MOJO_OS_VIP08="${TEST_VIP08}"
    export MOJO_OS_VIP09="${TEST_VIP09}"
    export MOJO_OS_VIP10="${TEST_VIP10}"
    export MOJO_OS_VIP11="${TEST_VIP11}"
    export MOJO_OS_VIP12="${TEST_VIP12}"
    export MOJO_OS_VIP13="${TEST_VIP13}"
    export MOJO_OS_VIP14="${TEST_VIP14}"
    export MOJO_OS_VIP15="${TEST_VIP15}"
    export MOJO_OS_VIP16="${TEST_VIP16}"
    export MOJO_OS_VIP17="${TEST_VIP17}"
    export MOJO_OS_VIP18="${TEST_VIP18}"
    export MOJO_OS_VIP19="${TEST_VIP19}"
    export MOJO_OS_VIP20="${TEST_VIP20}"
    export MOJO_OS_VIP="${TEST_VIP}"
    export MOJO_VIP_RANGE="${TEST_VIP_RANGE}"
    echo "  + Setting mojo network env vars:"
    echo "    MOJO_NAME_SERVER: $MOJO_NAME_SERVER"
    echo "    MOJO_NET_ID: $MOJO_NET_ID"
    echo "    MOJO_SUBNET_ID: $MOJO_SUBNET_ID"
    echo "    MOJO_CIDR_EXT: $MOJO_CIDR_EXT"
    echo "    MOJO_GATEWAY: $MOJO_GATEWAY"
    echo "    MOJO_FIP_RANGE: $MOJO_FIP_RANGE"
    echo "    MOJO_OS_VIP00: $MOJO_OS_VIP00"
    echo "    MOJO_OS_VIP01: $MOJO_OS_VIP01"
    echo "    MOJO_OS_VIP02: $MOJO_OS_VIP02"
    echo "    MOJO_OS_VIP03: $MOJO_OS_VIP03"
    echo "    MOJO_OS_VIP04: $MOJO_OS_VIP04"
    echo "    MOJO_OS_VIP05: $MOJO_OS_VIP05"
    echo "    MOJO_OS_VIP06: $MOJO_OS_VIP06"
    echo "    MOJO_OS_VIP07: $MOJO_OS_VIP07"
    echo "    MOJO_OS_VIP08: $MOJO_OS_VIP08"
    echo "    MOJO_OS_VIP09: $MOJO_OS_VIP09"
    echo "    MOJO_OS_VIP10: $MOJO_OS_VIP10"
    echo "    MOJO_OS_VIP11: $MOJO_OS_VIP11"
    echo "    MOJO_OS_VIP12: $MOJO_OS_VIP12"
    echo "    MOJO_OS_VIP13: $MOJO_OS_VIP13"
    echo "    MOJO_OS_VIP14: $MOJO_OS_VIP14"
    echo "    MOJO_OS_VIP15: $MOJO_OS_VIP15"
    echo "    MOJO_OS_VIP16: $MOJO_OS_VIP16"
    echo "    MOJO_OS_VIP17: $MOJO_OS_VIP17"
    echo "    MOJO_OS_VIP18: $MOJO_OS_VIP18"
    echo "    MOJO_OS_VIP19: $MOJO_OS_VIP19"
    echo "    MOJO_OS_VIP20: $MOJO_OS_VIP20"
    echo "    MOJO_OS_VIP: $MOJO_OS_VIP"
    echo "    MOJO_VIP_RANGE: $MOJO_VIP_RANGE"
}


function f_openstack_check_fail() {
# Check for failure touchfiles and fail if found
# Exclude '.bad' statuses such as tempest fail
# ie. pass the deploy even as tempest fails
  lso="$(ls $WORKSPACE -1)"
  if [[ "$lso" == *.failed* ]] || \
     [[ "$lso" == *.missing* ]]; then
    echo " ! Exiting 1."
    exit 1
  fi
}


function f_novarc_undercloud() {
  echo " . Sourcing undercloud novarc."
  stat -t ~/novarc
  . ~/novarc
}


function f_novarc_artifactcloud() {
  echo " . Sourcing artifactcloud novarc."
  . ~/novarc-artifactcloud
}
